@@ -1,5 +1,5 @@
 /* libmondo-archive.h
- * $Id: libmondo-archive.h 1885 2008-03-01 23:38:35Z bruno $
+ * $Id: libmondo-archive.h 3464 2015-09-10 09:52:41Z bruno $
  */
 
 int archive_this_fileset(char *filelist,
@@ -35,6 +35,4 @@ int write_iso_and_go_on(bool last_cd);
 int write_final_iso_if_necessary();
 int call_growisofs(char *destfile);
 int make_afioballs_and_images_SINGLETHREAD();
-int archive_this_fileset_with_star(
-								   char *filelist, char *fname, int setno);
 void setenv_mondo_share(void);
