@@ -1,4 +1,4 @@
-/* $Id: libmondo-string.c 3150 2013-06-21 04:48:23Z bruno $ */
+/* $Id: libmondo-string.c 3413 2015-08-09 04:15:26Z bruno $ */
 
 
 /**
@@ -8,6 +8,7 @@
 
 #include "my-stuff.h"
 #include "mr_mem.h"
+#include "mr_str.h"
 #include "mondostructures.h"
 #include "libmondo-string.h"
 #include "lib-common-externs.h"
@@ -16,7 +17,7 @@
 #include "libmondo-tools-EXT.h"
 
 /*@unused@*/
-//static char cvsid[] = "$Id: libmondo-string.c 3150 2013-06-21 04:48:23Z bruno $";
+//static char cvsid[] = "$Id: libmondo-string.c 3413 2015-08-09 04:15:26Z bruno $";
 
 extern int g_current_media_number;
 extern long long g_tape_posK;
@@ -50,6 +51,7 @@ char *build_partition_name(char *partition, const char *drive, int partno)
 	c = strrchr(partition, '/');
 	if (c && strncmp(c, "/disc", 5) == 0) {
 		/* yup it's devfs, return the "part" path */
+		/* This strcpy is safe */
 		strcpy(c + 1, "part");
 		p = c + 5;
 	} else {
@@ -64,20 +66,46 @@ char *build_partition_name(char *partition, const char *drive, int partno)
 		}
 	}
 	sprintf(p, "%d", partno);
-	return (partition);
+	return(partition);
 }
 
 
+/**
+ * Pad a string on both sides so it appears centered.
+ * @param in_out The string to be center-padded (modified).
+ * @param width The width of the final result.
+ */
+char *mr_center_string(char *in, int width)
+{
+	char *scratch = NULL;
+	char *out = NULL;
+	int i;						/* purpose */
+	int len;					/* purpose */
+	int mid;					/* purpose */
+	int x;						/* purpose */
 
+	assert(in != NULL);
+	assert(width > 2);
 
-
-
-
-
-
-
-
-
+	if (strlen(in) == 0) {
+		return(NULL);
+	}
+	/*  skip initial spaces */
+	mr_asprintf(scratch, "%s", in);
+	mr_strip_spaces(scratch);
+	len = (int)strlen(scratch);
+	mid = width / 2;
+	x = mid - len / 2;
+	for (i = 0; i < x; i++) {
+		mr_strcat(out," ");
+	}
+	mr_strcat(out, scratch);
+	mr_free(scratch);
+	for (i = x + len ; i < width - 1; i++) {
+      		mr_strcat(out," ");
+    	}
+	return(out);
+}
 
 /**
  * Pad a string on both sides so it appears centered.
@@ -101,7 +129,7 @@ void center_string(char *in_out, int width)
 	}
 	for (p = in_out; *p == ' '; p++);
 	strcpy(scratch, p);
-	strip_spaces (scratch);
+	strip_spaces(scratch);
 	len = (int) strlen(scratch);
 	mid = width / 2;
 	x = mid - len / 2;
@@ -117,22 +145,6 @@ void center_string(char *in_out, int width)
 }
 
 
-
-
-inline void turn_wildcard_chars_into_literal_chars(char *sout, char *sin)
-{
-	char *p, *q;
-
-	for (p = sin, q = sout; *p != '\0'; *(q++) = *(p++)) {
-		if (strchr("[]*?", *p)) {
-			*(q++) = '\\';
-		}
-	}
-	*q = *p;					// for the final '\0'
-}
-
-
-
 /**
  * Add commas every third place in @p input.
  * @param input The string to commarize.
@@ -154,8 +166,6 @@ char *commarize(char *input)
 		j = (int) strlen(pos_w_commas);
 		tmp[j - 6] = ',';
 		strcpy(tmp + j - 5, pos_w_commas + j - 6);
-//      tmp[j-2] = ',';
-//      strcpy(tmp+j-1, pos_w_commas+j-3);
 		strcpy(pos_w_commas, tmp);
 	}
 	if (strlen(tmp) > 3) {
@@ -171,13 +181,6 @@ char *commarize(char *input)
 }
 
 
-
-
-
-
-
-
-
 /**
  * Turn an entry from the RAID editor's disklist into a GUI-friendly string.
  * The format is: the device left-aligned and padded to 24 places, followed by a space and the
@@ -216,7 +219,7 @@ long friendly_sizestr_to_sizelong(char *incoming)
 {
 	long outval;
 	int i;
-	char *tmp;
+	char *tmp = NULL;
 	char ch;
 
 	assert_string_is_neither_NULL_nor_zerolength(incoming);
@@ -227,8 +230,8 @@ long friendly_sizestr_to_sizelong(char *incoming)
 	if (strchr(incoming, '.')) {
 		fatal_error("Please use integers only. No decimal points.");
 	}
-	malloc_string(tmp);
-	strcpy(tmp, incoming);
+
+	mr_asprintf(tmp, "%s", incoming);
 	i = (int) strlen(tmp);
 	if (tmp[i - 1] == 'B' || tmp[i - 1] == 'b') {
 		tmp[i - 1] = '\0';
@@ -237,6 +240,8 @@ long friendly_sizestr_to_sizelong(char *incoming)
 	ch = tmp[i];
 	tmp[i] = '\0';
 	outval = atol(tmp);
+	mr_free(tmp);
+
 	if (ch == 'g' || ch == 'G') {
 		outval = outval * 1024;
 	} else if (ch == 'k' || ch == 'K') {
@@ -252,11 +257,9 @@ long friendly_sizestr_to_sizelong(char *incoming)
 			("That sizespec is more than 1,208,925,819,614,629,174,706,176 bytes. You have a shocking amount of data. Please send a screenshot to the list :-)");
 		fatal_error("Integer overflow.");
 	} else if (ch != 'm' && ch != 'M') {
-		sprintf(tmp, "Re: parameter '%s' - bad multiplier ('%c')",
-				incoming, ch);
+		mr_asprintf(tmp, "Re: parameter '%s' - bad multiplier ('%c')", incoming, ch);
 		fatal_error(tmp);
 	}
-	paranoid_free(tmp);
 	return (outval);
 }
 
@@ -415,29 +418,31 @@ char *mountlist_entry_to_string(struct mountlist_itself *mountlist,
  * @param label The "blah blah" part in the middle. If you leave this blank
  * there will be a weird double space in the middle, so pass *something*.
  * @return The string containing "X blah blah disk(s)".
- * @note The returned string points to static storage and will be overwritten with each call.
+ * @note The returned string is dynamically allocated and should be freed by caller
  */
 char *number_of_disks_as_string(int noof_disks, char *label)
 {
 
-	/*@ buffers ********************************************************* */
-	static char output[MAX_STR_LEN];
+/*@ buffers ********************************************************* */
+char *output = NULL;
 
-	/*@ char     ******************************************************** */
-	char p;
+/*@ char     ******************************************************** */
+char p;
 
-	assert(label != NULL);
+assert(label != NULL);
 
-	if (noof_disks > 1) {
-		p = 's';
-	} else {
-		p = ' ';
-	}
-	sprintf(output, "%d %s disk%c", noof_disks, label, p);
-	while (strlen(output) < 14) {
-		strcat(output, " ");
-	}
-	return (output);
+if (noof_disks > 1) {
+	p = 's';
+} else {
+	p = ' ';
+}
+mr_asprintf(output, "%d %s disk%c", noof_disks, label, p);
+/* Useful ??
+while (strlen(output) < 14) {
+	strcat(output, " ");
+}
+*/
+return (output);
 }
 
 
@@ -506,6 +511,8 @@ char *number_to_text(int i)
  * @param token The token to be relaced with @p value.
  * @param value The value to replace @p token.
  */
+
+/* TODO: consider mr_strtok */
 void resolve_naff_tokens(char *output, char *ip, char *value, char *token)
 {
 	/*@ buffers *** */
@@ -515,9 +522,6 @@ void resolve_naff_tokens(char *output, char *ip, char *value, char *token)
 	char *p;
 
 	input = malloc(2000);
-	// BERLIOS: seems to cause invalid errors 
-	//assert_string_is_neither_NULL_nor_zerolength(ip);
-	assert_string_is_neither_NULL_nor_zerolength(token);
 	assert(value != NULL);
 
 	strcpy(output, ip);			/* just in case the token doesn't appear in string at all */
@@ -556,18 +560,18 @@ char *slice_fname(long bigfileno, long sliceno, char *path, char *s)
 
 	/*@ buffers **************************************************** */
 	static char output[MAX_STR_LEN];
-	static char suffix[MAX_STR_LEN];
+	char *suffix = NULL;
 
 	/*@ end vars *************************************************** */
 
 	assert_string_is_neither_NULL_nor_zerolength(path);
 	if (s[0] != '\0') {
-		sprintf(suffix, ".%s", s);
+		mr_asprintf(suffix, ".%s", s);
 	} else {
-		suffix[0] = '\0';
+		mr_asprintf(suffix, "");
 	}
-	sprintf(output, "%s/slice-%07ld.%05ld.dat%s", path, bigfileno, sliceno,
-			suffix);
+	sprintf(output, "%s/slice-%07ld.%05ld.dat%s", path, bigfileno, sliceno, suffix);
+	mr_free(suffix);
 	return (output);
 }
 
@@ -645,10 +649,6 @@ int strcmp_inc_numbers(char *stringA, char *stringB)
 	}
 	numA = atol(stringA + start_of_numbers_in_A);
 	numB = atol(stringB + start_of_numbers_in_B);
-	/*
-	   sprintf(tmp,"Comparing %s and %s --> %ld,%ld\n",stringA,stringB,numA,numB);
-	   log_to_screen(tmp);
-	 */
 	return ((int) (numA - numB));
 }
 
@@ -774,7 +774,6 @@ void strip_spaces2(char *in_out)
 	}
 	in_out[i] = '\0';
 	paranoid_free(tmp);
-/*  for(i=strlen(in_out); i>0 && in_out[i-1]<=32; i--) {in_out[i-1]='\0';} */
 }
 
 
@@ -816,7 +815,7 @@ char *truncate_to_drive_name(const char *partition)
 	char *trunc = NULL;
 
 	assert_string_is_neither_NULL_nor_zerolength(partition);
-	mr_asprintf(&trunc, "%s", partition);
+	mr_asprintf(trunc, "%s", partition);
 
 #ifdef __FreeBSD__
 
@@ -908,112 +907,115 @@ char *turn_raid_level_number_to_string(int raid_level)
  * @param out_reason If non-NULL, a descriptive reason for the difference will be copied here.
  * @return The severity (1-3).
  */
-int severity_of_difference(char *fn, char *out_reason)
-{
-	int sev;
-	char *reason;
-	char *filename;
+int severity_of_difference(char *fn, char **out_reason) {
 
-	malloc_string(reason);
-	malloc_string(filename);
-// out_reason might be null on purpose, so don't bomb if it is :) OK?
+	int sev = 3;
+	char *reason = NULL;
+	char *filename = NULL;
+
+	// out_reason might be null on purpose, so don't bomb if it is :) OK?
 	assert_string_is_neither_NULL_nor_zerolength(fn);
 	if (!strncmp(fn, MNT_RESTORING, strlen(MNT_RESTORING))) {
-		strcpy(filename, fn + strlen(MNT_RESTORING));
+		mr_asprintf(filename, "%s", fn + strlen(MNT_RESTORING));
 	} else if (fn[0] != '/') {
-		sprintf(filename, "/%s", fn);
+		mr_asprintf(filename, "/%s", fn);
 	} else {
-		strcpy(filename, fn);
+		mr_asprintf(filename, "%s", fn);
 	}
 
-	sev = 3;
-	sprintf(reason,
-			"Changed since backup. Consider running a differential backup in a day or two.");
+	mr_asprintf(reason, "Changed since backup. Consider running a differential backup in a day or two.");
+
 	if (!strncmp(filename, "/var/", 5)) {
 		sev = 2;
-		sprintf(reason,
-				"/var's contents will change regularly, inevitably.");
+		mr_free(reason);
+		mr_asprintf(reason, "/var's contents will change regularly, inevitably.");
 	}
 	if (!strncmp(filename, "/home", 5)) {
 		sev = 2;
-		sprintf(reason,
-				"It's in your /home directory. Therefore, it is important.");
+		mr_free(reason);
+		mr_asprintf(reason, "It's in your /home directory. Therefore, it is important.");
 	}
 	if (!strncmp(filename, "/usr/", 5)) {
 		sev = 3;
-		sprintf(reason,
-				"You may have installed/removed software during the backup.");
+		mr_free(reason);
+		mr_asprintf(reason, "You may have installed/removed software during the backup.");
 	}
 	if (!strncmp(filename, "/etc/", 5)) {
 		sev = 3;
-		sprintf(reason,
-				"Do not edit config files while backing up your PC.");
+		mr_free(reason);
+		mr_asprintf(reason, "Do not edit config files while backing up your PC.");
 	}
 	if (!strcmp(filename, "/etc/adjtime")
 		|| !strcmp(filename, "/etc/mtab")) {
 		sev = 1;
-		sprintf(reason, "This file changes all the time. It's OK.");
+		mr_free(reason);
+		mr_asprintf(reason, "This file changes all the time. It's OK.");
 	}
 	if (!strncmp(filename, "/root/", 6)) {
 		sev = 3;
-		sprintf(reason, "Were you compiling/editing something in /root?");
+		mr_free(reason);
+		mr_asprintf(reason, "Were you compiling/editing something in /root?");
 	}
 	if (!strncmp(filename, "/root/.", 7)) {
 		sev = 2;
-		sprintf(reason, "Temp or 'dot' files changed in /root.");
+		mr_free(reason);
+		mr_asprintf(reason, "Temp or 'dot' files changed in /root.");
 	}
 	if (!strncmp(filename, "/var/lib/", 9)) {
 		sev = 2;
-		sprintf(reason, "Did you add/remove software during backing?");
+		mr_free(reason);
+		mr_asprintf(reason, "Did you add/remove software during backing?");
 	}
 	if (!strncmp(filename, "/var/lib/rpm", 12)) {
 		sev = 3;
-		sprintf(reason, "Did you add/remove software during backing?");
+		mr_free(reason);
+		mr_asprintf(reason, "Did you add/remove software during backing?");
 	}
 	if (!strncmp(filename, "/var/lib/slocate", 16)) {
 		sev = 1;
-		sprintf(reason,
-				"The 'update' daemon ran during backup. This does not affect the integrity of your backup.");
+		mr_free(reason);
+		mr_asprintf(reason, "The 'update' daemon ran during backup. This does not affect the integrity of your backup.");
 	}
 	if (!strncmp(filename, "/var/log/", 9)
 		|| strstr(filename, "/.xsession")
 		|| !strcmp(filename + strlen(filename) - 4, ".log")) {
 		sev = 1;
-		sprintf(reason,
-				"Log files change frequently as the computer runs. Fret not.");
+		mr_free(reason);
+		mr_asprintf(reason, "Log files change frequently as the computer runs. Fret not.");
 	}
 	if (!strncmp(filename, "/var/spool", 10)) {
 		sev = 1;
-		sprintf(reason,
-				"Background processes or printers were active. This does not affect the integrity of your backup.");
+		mr_free(reason);
+		mr_asprintf(reason, "Background processes or printers were active. This does not affect the integrity of your backup.");
 	}
 	if (!strncmp(filename, "/var/spool/mail", 10)) {
 		sev = 2;
-		sprintf(reason, "Mail was sent/received during backup.");
+		mr_free(reason);
+		mr_asprintf(reason, "Mail was sent/received during backup.");
 	}
 	if (filename[strlen(filename) - 1] == '~') {
 		sev = 1;
-		sprintf(reason,
-				"Backup copy of another file which was modified recently.");
+		mr_free(reason);
+		mr_asprintf(reason, "Backup copy of another file which was modified recently.");
 	}
 	if (strstr(filename, "cache")) {
 		sev = 1;
-		sprintf(reason,
-				"Part of a cache of data. Caches change from time to time. Don't worry.");
+		mr_free(reason);
+		mr_asprintf(reason, "Part of a cache of data. Caches change from time to time. Don't worry.");
 	}
 	if (!strncmp(filename, "/var/run/", 9)
 		|| !strncmp(filename, "/var/lock", 8)
 		|| strstr(filename, "/.DCOPserver") || strstr(filename, "/.MCOP")
 		|| strstr(filename, "/.Xauthority")) {
 		sev = 1;
-		sprintf(reason,
-				"Temporary file (a lockfile, perhaps) used by software such as X or KDE to register its presence.");
+		mr_free(reason);
+		mr_asprintf(reason, "Temporary file (a lockfile, perhaps) used by software such as X or KDE to register its presence.");
 	}
 	if (out_reason) {
-		strcpy(out_reason, reason);
+		strcpy(*out_reason, reason);
 	}
-	paranoid_free(filename);
-	paranoid_free(reason);
+	mr_free(filename);
+	mr_free(reason);
 	return (sev);
 }
 
@@ -1062,53 +1064,46 @@ char *percent_media_full_comment()
 	int j;
 
 	/*@ buffers ******************************************* */
-	static char outstr[MAX_STR_LEN];
-	char *pos_w_commas, *tmp;
+	char * outstr = NULL;
+	char *pos_w_commas = NULL;
 	char *mds = NULL;
+	char *tmp = NULL;
 
 	assert(bkpinfo != NULL);
-	malloc_string(pos_w_commas);
-	malloc_string(tmp);
-	sprintf(tmp, "%lld", g_tape_posK);
-	strcpy(pos_w_commas, commarize(tmp));
 
-
-
-	if (bkpinfo->media_size <= 0)
-//    { fatal_error( "percentage_media_full_comment() - unknown media size"); }
-	{
-		sprintf(outstr, "Volume %d: %s kilobytes archived so far",
-				g_current_media_number, pos_w_commas);
+	if (bkpinfo->media_size <= 0) {
+		mr_asprintf(tmp, "%lld", g_tape_posK);
+		mr_asprintf(pos_w_commas, "%s", commarize(tmp));
+		mr_free(tmp);
+		mr_asprintf(outstr, "Volume %d: %s kilobytes archived so far", g_current_media_number, pos_w_commas);
+		mr_free(pos_w_commas);
 		return (outstr);
 	}
 
-/* update screen */
+	/* update screen */
 	if (IS_THIS_A_STREAMING_BACKUP(bkpinfo->backup_media_type)) {
 		percentage = (int) (g_tape_posK / 10 / bkpinfo->media_size);
 		if (percentage > 100) {
 			percentage = 100;
 		}
-		sprintf(outstr, "Volume %d: [", g_current_media_number);
+		mr_asprintf(outstr, "Volume %d: [", g_current_media_number);
 	} else {
-		percentage =
-			(int) (space_occupied_by_cd(bkpinfo->scratchdir) * 100 / 1024 / bkpinfo->media_size);
+		percentage = (int) (space_occupied_by_cd(bkpinfo->scratchdir) * 100 / 1024 / bkpinfo->media_size);
 		mds = media_descriptor_string(bkpinfo->backup_media_type);
-		sprintf(outstr, "%s %d: [", mds, g_current_media_number);
+		mr_asprintf(outstr, "%s %d: [", mds, g_current_media_number);
 		mr_free(mds);
 	}
 	for (j = 0; j < percentage; j += 5) {
-		strcat(outstr, "*");
+		mr_strcat(outstr, "*");
 	}
 	for (; j < 100; j += 5) {
-		strcat(outstr, ".");
+		mr_strcat(outstr, ".");
 	}
-	j = (int) strlen(outstr);
-	sprintf(outstr + j, "] %d%% used", percentage);
-	paranoid_free(pos_w_commas);
-	paranoid_free(tmp);
+	mr_strcat(outstr, "] %d%% used", percentage);
 	return (outstr);
 }
 
+
 /**
  * Get a string form of @p type_of_bkp.
  * @param type_of_bkp The backup type to stringify.
@@ -1121,34 +1116,34 @@ char *media_descriptor_string(t_bkptype type_of_bkp) {
 
 	switch (type_of_bkp) {
 	case dvd:
-		mr_asprintf(&type_of_backup, "DVD");
+		mr_asprintf(type_of_backup, "DVD");
 		break;
 	case cdr:
-		mr_asprintf(&type_of_backup, "CDR");
+		mr_asprintf(type_of_backup, "CDR");
 		break;
 	case cdrw:
-		mr_asprintf(&type_of_backup, "CDRW");
+		mr_asprintf(type_of_backup, "CDRW");
 		break;
 	case tape:
-		mr_asprintf(&type_of_backup, "tape");
+		mr_asprintf(type_of_backup, "tape");
 		break;
 	case cdstream:
-		mr_asprintf(&type_of_backup, "CDR");
+		mr_asprintf(type_of_backup, "CDR");
 		break;
 	case udev:
-		mr_asprintf(&type_of_backup, "udev");
+		mr_asprintf(type_of_backup, "udev");
 		break;
 	case iso:
-		mr_asprintf(&type_of_backup, "ISO");
+		mr_asprintf(type_of_backup, "ISO");
 		break;
 	case netfs:
-		mr_asprintf(&type_of_backup, "%s", bkpinfo->netfs_proto);
+		mr_asprintf(type_of_backup, "%s", bkpinfo->netfs_proto);
 		break;
     case usb:
-        mr_asprintf(&type_of_backup, "USB");
+        mr_asprintf(type_of_backup, "USB");
         break;
 	default:
-		mr_asprintf(&type_of_backup, "ISO");
+		mr_asprintf(type_of_backup, "ISO");
 	}
 	return (type_of_backup);
 }
