@@ -4,7 +4,7 @@
     begin                : Sat Apr 20 2002
     copyright            : (C) 2002 by Stan Benoit
     email                : troff@nakedsoul.org
-    cvsid                : $Id: mondoprep.h 3042 2012-10-07 18:44:28Z bruno $
+    cvsid                : $Id: mondoprep.h 3300 2014-06-08 08:51:42Z bruno $
  ***************************************************************************/
 
 /***************************************************************************
@@ -68,7 +68,7 @@ int format_device(char *, char *, struct raidlist_itself *);
 int partition_drive(struct mountlist_itself *, char *);
 int partition_everything(struct mountlist_itself *);
 int do_my_funky_lvm_stuff(bool, bool);
-int which_format_command_do_i_need(char *, char *);
+char *which_format_command_do_i_need(char *);
 int make_dummy_partitions(FILE *, char *, int);
 int make_list_of_drives(struct mountlist_itself *,
 						char drivelist[ARBITRARY_MAXIMUM][MAX_STR_LEN]);
